Trend <- function(var, posTR = 2, interval = 1, pldegree = 1, parallel = FALSE, ncores = NULL) {
  if (!is.null(dim(var))) {
    margins <- c(1:length(dim(var)))[-posTR]
    if (parallel == TRUE) {
      if (is.null(ncores)) {
        ncores <- availableCores() - 1
      } else {
        ncores <- min(availableCores() - 1, ncores)
      }
      registerDoMC(ncores)
    }
    output <- Apply(list(var), margins = list(margins), AtomicFun = ".Trend",
                    interval, pldegree, parallel = parallel)
    dimnames(output) <- NULL
    output <- aperm(output, c(2 : length(dim(output)), 1))
    length_dim <- dim(output)[length(dim(output))]
    length_x <- length_dim  - dim(var)[posTR]
    output <- list(trend = Subset(output, along = length(dim(output)),
                                  indices = c(1 : (length_dim  - length_x + 1))),
                   detrended = Subset(output, along = length(dim(output)),
                                      indices = c(c(length_x + 1) : length_dim)))
  } else {
    output <- .Trend(var, interval = interval, pldegree = pldegree)
    output <- list(trend = output[c(1 : (length(output) - length(var)))],
                   detrended = output[(length(output) - length(var) + 1) : length(output)])
  }
  return(output)
}

.Trend <- function(data, interval = interval, pldegree = pldegree) {
  if (length(data) < (pldegree + 1)) {
    stop("selected polynomial degree (pldegree) is too high")
  }
  if (length(sort(data)) > 0) {
    mon <- seq(data) * interval
    lm.out <- lm(data ~ poly(mon, degree = pldegree, raw = TRUE), na.action = na.omit)
    trend <- detrended <- c()
    for (jj in (pldegree + 1):1) {
      jj1 <- 1 + (pldegree + 1) - jj
      if ( (jj==1) & (pldegree==1) ) {
        trend <- c(trend, lm.out$coefficients[jj], confint(lm.out)[jj, 1],
                   confint(lm.out)[jj, 2])
      } else {
        trend <- c(trend, confint(lm.out)[jj, 1], lm.out$coefficients[jj],
                   confint(lm.out)[jj, 2])
      }
    }
    names(trend) <- NULL
    detrended[is.na(data) == FALSE] <- data[is.na(data) == FALSE] - lm.out$fitted.values
  }
  return(c(trend, detrended))
}